<?php

namespace Drupal\aws_cloud\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a block displaying unused volumes.
 *
 * @Block(
 *   id = "aws_cloud_unused_volumes_block",
 *   admin_label = @Translation("Unused Volumes"),
 *   category = @Translation("AWS Cloud")
 * )
 */
class UnusedVolumesBlock extends BlockBase implements ContainerFactoryPluginInterface {

  use StringTranslationTrait;

  /**
   * Stores the configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Creates a ResourcesBlock instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ConfigFactoryInterface $config_factory,
    AccountInterface $current_user
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    return $this->buildVolumeList();
  }

  /**
   * Build a list of unused volumes.
   *
   * @return array
   *   A set of render arrays.
   */
  private function buildVolumeList() {
    $build = [];
    $volumes = $this->getUnusedVolumes();
    if (count($volumes)) {
      $urls = [];
      /* @var \Drupal\aws_cloud\Entity\Ec2\Volume $volume */
      foreach ($volumes as $volume) {
        $days_running = $volume->daysRunning();
        $link_text = $this->t('@volume (%unused @days)', [
          '@volume' => $volume->getName(),
          '%unused' => $days_running,
          '@days' => $this->formatPlural($days_running, 'day', 'days'),
        ]);
        $urls[] = $volume->toLink($link_text);
      }
      $unused_days = $this->configFactory->get('aws_cloud.settings')->get('aws_cloud_unused_volume_criteria');
      $build[] = [
        '#markup' => $this->t('The following volumes have been unused for more than %num days', ['%num' => $unused_days]),
      ];
      $build[] = [
        '#theme' => 'item_list',
        '#items' => $urls,
      ];
    }
    else {
      $build = [
        '#markup' => $this->t('Great job! You have no unused volumes.'),
      ];
    }
    return $build;
  }

  /**
   * Get a list of unused volumes.
   *
   * @return array
   *   Array of unused volumes.
   */
  private function getUnusedVolumes() {
    $volumes = aws_cloud_get_unused_volumes();
    if (!$this->currentUser->hasPermission('view any aws cloud volume')) {
      /* @var \Drupal\aws_cloud\Entity\Ec2\Volume $volume */
      foreach ($volumes as $key => $volume) {
        // Only return volumes the user has access to.
        if ($volume->getOwnerId() !== $this->currentUser->id()) {
          unset($volumes[$key]);
        }
      }
    }
    return $volumes;
  }

}
