<?php

namespace Drupal\Tests\k8s\Functional;

/**
 * Tests K8s replica set.
 *
 * @group k8s
 */
class K8sReplicaSetTest extends K8sTestCase {

  const K8S_REPLICA_SET_REPEAT_COUNT = 3;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    $namespaces = $this->createNamespacesRandomTestFormData();
    $this->createNamespaceTestEntity($namespaces[0]);
    $this->namespace = $namespaces[0]['name'];

    return [
      'list k8s replica set',
      'view k8s replica set',
      'edit k8s replica set',
      'add k8s replica set',
      'delete k8s replica set',
      'view k8s namespace ' . $this->namespace,
    ];
  }

  /**
   * Tests CRUD for Replica Set.
   */
  public function testReplicaSet() {

    $cloud_context = $this->cloudContext;

    // List Replica Set for K8s.
    $this->drupalGet("/clouds/k8s/$cloud_context/replica_set");
    $this->assertResponse(200);

    // Add a new Replica Set.
    $add = $this->createReplicaSetTestFormData(self::K8S_REPLICA_SET_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_REPLICA_SET_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $this->addReplicaSetMockData($add[$i]);

      $num = $i + 1;
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/replica_set/add",
        $add[$i]['post_data'],
        t('Save')
      );
      $this->assertResponse(200);
      $this->assertText(t('The Kubernetes Replica Set "@name" has been created.', ['@name' => $add[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/replica_set");
      $this->assertResponse(200);
      $this->assertText($add[$i]['name']);
    }

    // Edit a Replica Set.
    $edit = $this->createReplicaSetTestFormData(self::K8S_REPLICA_SET_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_REPLICA_SET_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->updateReplicaSetMockData($edit[$i]);

      unset($edit[$i]['post_data']['namespace']);
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/replica_set/$num/edit",
        $edit[$i]['post_data'],
        t('Save')
      );
      $this->assertResponse(200);
      $this->assertText(t(
        'The Kubernetes Replica Set "@name" has been saved.',
        ['@name' => $add[$i]['name']]
      ));
    }

    // Delete Replica Set.
    for ($i = 0; $i < self::K8S_REPLICA_SET_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->deleteReplicaSetMockData($add[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/replica_set/$num/delete",
        [],
        t('Delete')
      );
      $this->assertResponse(200);
      $this->assertText(t(
        'The Kubernetes Replica Set "@name" has been deleted.',
        ['@name' => $add[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/replica_set");
      $this->assertResponse(200);
      $this->assertNoText($add[$i]['name']);
    }
  }

  /**
   * Tests deleting replica sets with bulk operation.
   */
  public function testReplicaSetBulk() {
    $cloud_context = $this->cloudContext;
    for ($i = 0; $i < self::K8S_REPLICA_SET_REPEAT_COUNT; $i++) {
      // Create Replica Sets.
      $replica_sets = $this->createReplicaSetsRandomTestFormData($this->namespace);
      $entities = [];
      foreach ($replica_sets as $replica_set) {
        $entities[] = $this->createReplicaSetTestEntity($replica_set);
      }
      $this->deleteReplicaSetMockData($replica_sets[0]);
      $this->doTestEntityBulk('replica_set', $entities);
    }
  }

}
